@@ -1158,8 +1158,9 @@ class Term implements REFlags{
                case 'o':   // oct 2- or 3-digit number -> char
                   int oct=0;
                   for(;;){
-                     char d=data[i++];
+                     char d=data[i];
                      if(d>='0' && d<='7'){
+                         i++;
                         oct*=8;
                         oct+=d-'0';
                         if(oct>0xffff) break;
@@ -1182,6 +1183,7 @@ class Term implements REFlags{
                      }
                      else break;
                   }
+                  i--;
                   c=(char)dec;
                   break;
                   
