@@ -610,6 +610,9 @@ public final class Ruby {
     private void init() {
         ThreadContext tc = getCurrentContext();
 
+        defineGlobalVERBOSE();
+        defineGlobalDEBUG();
+
         javaSupport = new JavaSupport(this);
 
         tc.preInitCoreClasses();
@@ -678,9 +681,7 @@ public final class Ruby {
         registerBuiltin("thread.so", new LateLoadingLibrary("thread", "org.jruby.libraries.ThreadLibrary", getJRubyClassLoader()));
         registerBuiltin("openssl.so", new Library() {
                 public void load(Ruby runtime) throws IOException {
-                    runtime.getModule("Kernel").callMethod(runtime.getCurrentContext(),"require",runtime.newString("rubygems"));
-                    runtime.getTopSelf().callMethod(runtime.getCurrentContext(),"gem",runtime.newString("jruby-openssl"));
-                    runtime.getModule("Kernel").callMethod(runtime.getCurrentContext(),"require",runtime.newString("openssl.rb"));
+                    runtime.getLoadService().require("jruby/openssl/stub");
                 }
             });
         registerBuiltin("digest.so", new LateLoadingLibrary("digest", "org.jruby.libraries.DigestLibrary", getJRubyClassLoader()));
@@ -1890,4 +1891,43 @@ public final class Ruby {
     public static void setSecurityRestricted(boolean restricted) {
         securityRestricted = restricted;
     }
+    
+    private void defineGlobalVERBOSE() {
+        // $VERBOSE can be true, false, or nil.  Any non-false-nil value will get stored as true
+        getGlobalVariables().define("$VERBOSE", new IAccessor() {
+            public IRubyObject getValue() {
+                return getVerbose();
+            }
+
+            public IRubyObject setValue(IRubyObject newValue) {
+                if (newValue.isNil()) {
+                    setVerbose(newValue);
+                } else {
+                    setVerbose(newBoolean(newValue != getFalse()));
+                }
+
+                return newValue;
+            }
+        });
+    }
+
+    private void defineGlobalDEBUG() {
+        IAccessor d = new IAccessor() {
+            public IRubyObject getValue() {
+                return getDebug();
+            }
+
+            public IRubyObject setValue(IRubyObject newValue) {
+                if (newValue.isNil()) {
+                    setDebug(newValue);
+                } else {
+                    setDebug(newBoolean(newValue != getFalse()));
+                }
+
+                return newValue;
+            }
+            };
+        getGlobalVariables().define("$DEBUG", d);
+        getGlobalVariables().define("$-d", d);
+    }
 }
