@@ -44,6 +44,7 @@ import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.IOException;
 import java.math.BigInteger;
+import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.Iterator;
 
@@ -130,7 +131,9 @@ public class RubyKernel {
         module.defineModuleFunction("scan", callbackFactory.getSingletonMethod("scan", IRUBY_OBJECT));
         module.defineFastModuleFunction("select", callbackFactory.getFastOptSingletonMethod("select"));
         module.defineModuleFunction("set_trace_func", callbackFactory.getSingletonMethod("set_trace_func", IRUBY_OBJECT));
-        module.defineFastModuleFunction("sleep", callbackFactory.getFastSingletonMethod("sleep", IRUBY_OBJECT));
+        module.defineModuleFunction("trace_var", callbackFactory.getOptSingletonMethod("trace_var"));
+        module.defineModuleFunction("untrace_var", callbackFactory.getOptSingletonMethod("untrace_var"));
+        module.defineFastModuleFunction("sleep", callbackFactory.getFastOptSingletonMethod("sleep"));
         module.defineFastModuleFunction("split", callbackFactory.getFastOptSingletonMethod("split"));
         module.defineFastModuleFunction("sprintf", callbackFactory.getFastOptSingletonMethod("sprintf"));
         module.defineFastModuleFunction("srand", callbackFactory.getFastOptSingletonMethod("srand"));
@@ -143,7 +146,7 @@ public class RubyKernel {
         module.defineFastModuleFunction("test", callbackFactory.getFastOptSingletonMethod("test"));
         module.defineModuleFunction("throw", callbackFactory.getOptSingletonMethod("rbThrow"));
         // TODO: Implement Kernel#trace_var
-        module.defineModuleFunction("trap", callbackFactory.getOptSingletonMethod("trap"));
+        module.definePublicModuleFunction("trap", callbackFactory.getOptSingletonMethod("trap"));
         // TODO: Implement Kernel#untrace_var
         module.defineFastModuleFunction("warn", callbackFactory.getFastSingletonMethod("warn", IRUBY_OBJECT));
         
@@ -173,21 +176,23 @@ public class RubyKernel {
         module.defineFastPublicModuleFunction("class", objectCallbackFactory.getFastMethod("type"));
         module.defineFastPublicModuleFunction("inspect", objectCallbackFactory.getFastMethod("inspect"));
         module.defineFastPublicModuleFunction("=~", objectCallbackFactory.getFastMethod("match", IRUBY_OBJECT));
-        module.defineFastPublicModuleFunction("clone", objectCallbackFactory.getFastMethod("rbClone"));
+        module.definePublicModuleFunction("clone", objectCallbackFactory.getMethod("rbClone"));
         module.defineFastPublicModuleFunction("display", objectCallbackFactory.getFastOptMethod("display"));
         module.defineFastPublicModuleFunction("extend", objectCallbackFactory.getFastOptMethod("extend"));
         module.defineFastPublicModuleFunction("freeze", objectCallbackFactory.getFastMethod("freeze"));
         module.defineFastPublicModuleFunction("frozen?", objectCallbackFactory.getFastMethod("frozen"));
         module.defineFastModuleFunction("initialize_copy", objectCallbackFactory.getFastMethod("initialize_copy", IRUBY_OBJECT));
         module.definePublicModuleFunction("instance_eval", objectCallbackFactory.getOptMethod("instance_eval"));
+        module.definePublicModuleFunction("instance_exec", objectCallbackFactory.getOptMethod("instance_exec"));
         module.defineFastPublicModuleFunction("instance_of?", objectCallbackFactory.getFastMethod("instance_of", IRUBY_OBJECT));
         module.defineFastPublicModuleFunction("instance_variables", objectCallbackFactory.getFastMethod("instance_variables"));
         module.defineFastPublicModuleFunction("instance_variable_get", objectCallbackFactory.getFastMethod("instance_variable_get", IRUBY_OBJECT));
         module.defineFastPublicModuleFunction("instance_variable_set", objectCallbackFactory.getFastMethod("instance_variable_set", IRUBY_OBJECT, IRUBY_OBJECT));
+        module.defineFastPublicModuleFunction("instance_variable_defined?", objectCallbackFactory.getFastMethod("instance_variable_defined_p", IRUBY_OBJECT));
         module.defineFastPublicModuleFunction("method", objectCallbackFactory.getFastMethod("method", IRUBY_OBJECT));
         module.defineFastPublicModuleFunction("methods", objectCallbackFactory.getFastOptMethod("methods"));
-        module.defineFastPublicModuleFunction("private_methods", objectCallbackFactory.getFastMethod("private_methods"));
-        module.defineFastPublicModuleFunction("protected_methods", objectCallbackFactory.getFastMethod("protected_methods"));
+        module.defineFastPublicModuleFunction("private_methods", objectCallbackFactory.getFastOptMethod("private_methods"));
+        module.defineFastPublicModuleFunction("protected_methods", objectCallbackFactory.getFastOptMethod("protected_methods"));
         module.defineFastPublicModuleFunction("public_methods", objectCallbackFactory.getFastOptMethod("public_methods"));
         module.defineFastModuleFunction("remove_instance_variable", objectCallbackFactory.getMethod("remove_instance_variable", IRUBY_OBJECT));
         module.defineFastPublicModuleFunction("respond_to?", objectCallbackFactory.getFastOptMethod("respond_to"));
@@ -204,77 +209,127 @@ public class RubyKernel {
     }
 
     public static IRubyObject at_exit(IRubyObject recv, Block block) {
-        return recv.getRuntime().pushExitBlock(recv.getRuntime().newProc(false, block));
+        return recv.getRuntime().pushExitBlock(recv.getRuntime().newProc(Block.Type.PROC, block));
     }
 
     public static IRubyObject autoload_p(final IRubyObject recv, IRubyObject symbol) {
-        String name = symbol.asSymbol();
-        if (recv instanceof RubyModule) {
-            name = ((RubyModule)recv).getName() + "::" + name;
-        }
+        RubyModule module = recv instanceof RubyModule ? (RubyModule) recv : recv.getRuntime().getObject();
+        String name = module.getName() + "::" + symbol.asSymbol();
         
         IAutoloadMethod autoloadMethod = recv.getRuntime().getLoadService().autoloadFor(name);
-        if(autoloadMethod == null) return recv.getRuntime().getNil();
+        if (autoloadMethod == null) return recv.getRuntime().getNil();
 
         return recv.getRuntime().newString(autoloadMethod.file());
     }
 
     public static IRubyObject autoload(final IRubyObject recv, IRubyObject symbol, final IRubyObject file) {
-        final LoadService loadService = recv.getRuntime().getLoadService();
+        Ruby runtime = recv.getRuntime(); 
+        final LoadService loadService = runtime.getLoadService();
         final String baseName = symbol.asSymbol();
-        String nm = baseName;
-        if(recv instanceof RubyModule) {
-            nm = ((RubyModule)recv).getName() + "::" + nm;
-        }
+        final RubyModule module = recv instanceof RubyModule ? (RubyModule) recv : runtime.getObject();
+        String nm = module.getName() + "::" + baseName;
+        
+        IRubyObject undef = runtime.getUndef();
+        IRubyObject existingValue = module.getInstanceVariable(baseName); 
+        if (existingValue != null && existingValue != undef) return runtime.getNil();
+        
+        module.setInstanceVariable(baseName, undef);
+        
         loadService.addAutoload(nm, new IAutoloadMethod() {
-                public String file() {
-                    return file.toString();
-                }
+            public String file() {
+                return file.toString();
+            }
             /**
              * @see org.jruby.runtime.load.IAutoloadMethod#load(Ruby, String)
              */
             public IRubyObject load(Ruby runtime, String name) {
-                loadService.require(file.toString());
-                if(recv instanceof RubyModule) {
-                    return ((RubyModule)recv).getConstant(baseName);
-                }
-                return runtime.getObject().getConstant(baseName);
+                boolean required = loadService.require(file.toString());
+                
+                // File to be loaded by autoload has already been or is being loaded.
+                if (!required) return null;
+                
+                return module.getConstant(baseName);
             }
         });
-        return recv;
+        return runtime.getNil();
     }
 
     public static IRubyObject method_missing(IRubyObject recv, IRubyObject[] args, Block block) {
         Ruby runtime = recv.getRuntime();
-        if (args.length == 0) {
-            throw recv.getRuntime().newArgumentError("no id given");
+
+        if (args.length == 0 || !(args[0] instanceof RubySymbol)) throw runtime.newArgumentError("no id given");
+        
+        String name = args[0].asSymbol();
+        ThreadContext context = runtime.getCurrentContext();
+        Visibility lastVis = context.getLastVisibility();
+        CallType lastCallType = context.getLastCallType();
+
+        String format = null;
+
+        boolean noMethod = true; // NoMethodError
+
+        if (lastVis == Visibility.PRIVATE) {
+            format = "private method `%s' called for %s";
+        } else if (lastVis == Visibility.PROTECTED) {
+            format = "protected method `%s' called for %s";
+        } else if (lastCallType == CallType.VARIABLE) {
+            format = "undefined local variable or method `%s' for %s";
+            noMethod = false; // NameError
+        } else if (lastCallType == CallType.SUPER) {
+            format = "super: no superclass method `%s'";
         }
 
-        String name = args[0].asSymbol();
+        if (format == null) format = "undefined method `%s' for %s";
+
         String description = null;
-        if("inspect".equals(name) || "to_s".equals(name)) {
-            description = recv.anyToString().toString();
-        } else {
-            description = recv.inspect().toString();
-        }
-        boolean noClass = description.length() > 0 && description.charAt(0) == '#';
-        ThreadContext tc = runtime.getCurrentContext();
-        Visibility lastVis = tc.getLastVisibility();
-        if(null == lastVis) {
-            lastVis = Visibility.PUBLIC;
-        }
-        CallType lastCallType = tc.getLastCallType();
-        String format = lastVis.errorMessageFormat(lastCallType, name);
-        // FIXME: Modify sprintf to accept Object[] as well...
-        String msg = Sprintf.sprintf(runtime.newString(format), 
-                runtime.newArray(new IRubyObject[] { 
-                        runtime.newString(name), 
-                        runtime.newString(description),
-                        runtime.newString(noClass ? "" : ":"), 
-                        runtime.newString(noClass ? "" : recv.getType().getName())
-                })).toString();
         
-        throw lastCallType == CallType.VARIABLE ? runtime.newNameError(msg, name) : runtime.newNoMethodError(msg, name);
+        if (recv.isNil()) {
+            description = "nil";
+        } else if (recv instanceof RubyBoolean && recv.isTrue()) {
+            description = "true";
+        } else if (recv instanceof RubyBoolean && !recv.isTrue()) {
+            description = "false";
+        } else {
+            if (name.equals("inspect") || name.equals("to_s")) {
+                description = recv.anyToString().toString();
+            } else {
+                IRubyObject d;
+                try {
+                    d = recv.callMethod(context, "inspect");
+                    if (d.getMetaClass() == recv.getMetaClass() || (d instanceof RubyString && ((RubyString)d).length().getLongValue() > 65)) {
+                        d = recv.anyToString();
+                    }
+                } catch (JumpException je) {
+                    d = recv.anyToString();
+                }
+                description = d.toString();
+            }
+        }
+        if (description.length() == 0 || (description.length() > 0 && description.charAt(0) != '#')) {
+            description = description + ":" + recv.getMetaClass().getRealClass().getName();            
+        }
+        
+        IRubyObject[]exArgs = new IRubyObject[noMethod ? 3 : 2];
+
+        RubyArray arr = runtime.newArray(args[0], runtime.newString(description));
+        RubyString msg = runtime.newString(Sprintf.sprintf(runtime.newString(format), arr).toString());
+        
+        if (recv.isTaint()) msg.setTaint(true);
+
+        exArgs[0] = msg;
+        exArgs[1] = args[0];
+
+        RubyClass exc;
+        if (noMethod) {
+            IRubyObject[]NMEArgs = new IRubyObject[args.length - 1];
+            System.arraycopy(args, 1, NMEArgs, 0, NMEArgs.length);
+            exArgs[2] = runtime.newArrayNoCopy(NMEArgs);
+            exc = runtime.getClass("NoMethodError");
+        } else {
+            exc = runtime.getClass("NameError");
+        }
+        
+        throw new RaiseException((RubyException)exc.newInstance(exArgs, Block.NULL_BLOCK));
     }
 
     public static IRubyObject open(IRubyObject recv, IRubyObject[] args, Block block) {
@@ -363,14 +418,25 @@ public class RubyKernel {
     }
     
     public static IRubyObject new_integer(IRubyObject recv, IRubyObject object) {
-        ThreadContext context = recv.getRuntime().getCurrentContext();
-        
-        if(object instanceof RubyString) {
+        if (object instanceof RubyFloat) {
+            double val = ((RubyFloat)object).getDoubleValue(); 
+            if (val <= (double) RubyFixnum.MAX && val >= (double) RubyFixnum.MIN) {
+                IRubyObject tmp = ((RubyObject)object).convertToType(recv.getRuntime().getClass("Integer"), MethodIndex.TO_INT, "to_int", false);
+                if (tmp.isNil()) return ((RubyObject)object).convertToType(recv.getRuntime().getClass("Integer"), MethodIndex.TO_I, "to_i", true);
+                return tmp;
+            }
+            return RubyNumeric.dbl2num(recv.getRuntime(),((RubyFloat)object).getDoubleValue());            
+        } else if (object instanceof RubyFixnum || object instanceof RubyBignum) {
+            return object;
+        } else if (object instanceof RubyString) {
             return RubyNumeric.str2inum(recv.getRuntime(),(RubyString)object,0,true);
-                    }
-        return object.callMethod(context,MethodIndex.TO_I, "to_i");
+        }
+        
+        IRubyObject tmp = ((RubyObject)object).convertToType(recv.getRuntime().getClass("Integer"), MethodIndex.TO_INT, "to_int", false);
+        if (tmp.isNil()) return ((RubyObject)object).convertToType(recv.getRuntime().getClass("Integer"), MethodIndex.TO_I, "to_i", true);
+        return tmp;
     }
-    
+
     public static IRubyObject new_string(IRubyObject recv, IRubyObject object) {
         return object.callMethod(recv.getRuntime().getCurrentContext(), MethodIndex.TO_S, "to_s");
     }
@@ -534,15 +600,32 @@ public class RubyKernel {
         return RubyIO.select_static(recv.getRuntime(), args);
     }
 
-    public static IRubyObject sleep(IRubyObject recv, IRubyObject seconds) {
-        long milliseconds = (long) (seconds.convertToFloat().getDoubleValue() * 1000);
+    public static IRubyObject sleep(IRubyObject recv, IRubyObject[] args) {
+        long milliseconds;
+
+        if (args.length == 0) {
+            // Zero sleeps forever
+            milliseconds = 0;
+        } else {
+            milliseconds = (long) (args[0].convertToFloat().getDoubleValue() * 1000);
+            if (milliseconds < 0) {
+                throw recv.getRuntime().newArgumentError("time interval must be positive");
+            } else if (milliseconds == 0) {
+                // Explicit zero in MRI returns immediately
+                return recv.getRuntime().newFixnum(0);
+            }
+        }
         long startTime = System.currentTimeMillis();
         
         RubyThread rubyThread = recv.getRuntime().getThreadService().getCurrentContext().getThread();
         
-        try {
-            rubyThread.sleep(milliseconds);
-        } catch (InterruptedException iExcptn) {
+        while(milliseconds > 0) {
+            long loopStartTime = System.currentTimeMillis();
+            try {
+                rubyThread.sleep(milliseconds);
+            } catch (InterruptedException iExcptn) {
+            }
+            milliseconds -= (System.currentTimeMillis() - loopStartTime);
         }
 
         return recv.getRuntime().newFixnum(
@@ -706,8 +789,8 @@ public class RubyKernel {
                 file = args[2].toString();
             }
         }
-        // FIXME: line number is not supported yet
-        //int line = args.length > 3 ? RubyNumeric.fix2int(args[3]) : 1;
+
+        int line = args.length > 3 ? RubyNumeric.fix2int(args[3]) - 1 : 1;
 
         recv.getRuntime().checkSafeString(src);
         ThreadContext context = recv.getRuntime().getCurrentContext();
@@ -716,7 +799,7 @@ public class RubyKernel {
             scope = RubyBinding.newBindingForEval(recv.getRuntime());
         }
         
-        return recv.evalWithBinding(context, src, scope, file);
+        return recv.evalWithBinding(context, src, scope, file, line);
     }
 
     public static IRubyObject callcc(IRubyObject recv, IRubyObject[] args, Block block) {
@@ -734,7 +817,7 @@ public class RubyKernel {
             throw recv.getRuntime().newArgumentError("negative level(" + level + ')');
         }
         
-        return recv.getRuntime().getCurrentContext().createBacktrace(level, false);
+        return ThreadContext.createBacktraceFromFrames(recv.getRuntime(), recv.getRuntime().getCurrentContext().createBacktrace(level, false));
     }
 
     public static IRubyObject rbCatch(IRubyObject recv, IRubyObject tag, Block block) {
@@ -757,20 +840,16 @@ public class RubyKernel {
         Ruby runtime = recv.getRuntime();
 
         String tag = args[0].asSymbol();
-        String[] catches = runtime.getCurrentContext().getActiveCatches();
+        ThreadContext context = runtime.getCurrentContext();
+        String[] catches = context.getActiveCatches();
 
-        String message = "uncaught throw '" + tag + '\'';
+        String message = "uncaught throw `" + tag + "'";
 
         //Ordering of array traversal not important, just intuitive
         for (int i = catches.length - 1 ; i >= 0 ; i--) {
             if (tag.equals(catches[i])) {
                 //Catch active, throw for catch to handle
-                JumpException je = recv.getRuntime().getCurrentContext().controlException;
-                je.setJumpType(JumpException.JumpType.ThrowJump);
-
-                je.setTarget(tag);
-                je.setValue(args.length > 1 ? args[1] : runtime.getNil());
-                throw je;
+                throw context.prepareJumpException(JumpException.JumpType.ThrowJump, tag, args.length > 1 ? args[1] : runtime.getNil());
             }
         }
 
@@ -779,16 +858,16 @@ public class RubyKernel {
     }
 
     public static IRubyObject trap(IRubyObject recv, IRubyObject[] args, Block block) {
-        // FIXME: We can probably fake some basic signals, but obviously can't do everything. For now, stub.
-        return recv.getRuntime().getNil();
+        recv.getRuntime().getLoadService().require("jsignal");
+        return recv.callMethod(recv.getRuntime().getCurrentContext(), "__jtrap", args, CallType.FUNCTIONAL, block);
     }
     
     public static IRubyObject warn(IRubyObject recv, IRubyObject message) {
         Ruby runtime = recv.getRuntime();
-        IRubyObject out = runtime.getObject().getConstant("STDERR");
-        RubyIO io = (RubyIO) out.convertToType(runtime.getClass("IO"), 0, "to_io", true); 
-
-        io.puts(new IRubyObject[] { message });
+        if (!runtime.getVerbose().isNil()) {
+            IRubyObject out = runtime.getGlobalVariables().get("$stderr");
+            out.callMethod(runtime.getCurrentContext(), "puts", new IRubyObject[] { message });
+        }
         return recv.getRuntime().getNil();
     }
 
@@ -803,6 +882,60 @@ public class RubyKernel {
         return trace_func;
     }
 
+    public static IRubyObject trace_var(IRubyObject recv, IRubyObject[] args, Block block) {
+        if (args.length == 0) throw recv.getRuntime().newArgumentError(0, 1);
+        RubyProc proc = null;
+        String var = null;
+        
+        if (args.length > 1) {
+            var = args[0].toString();
+        }
+        
+        if (var.charAt(0) != '$') {
+            // ignore if it's not a global var
+            return recv.getRuntime().getNil();
+        }
+        
+        if (args.length == 1) {
+            proc = RubyProc.newProc(recv.getRuntime(), block, Block.Type.PROC);
+        }
+        if (args.length == 2) {
+            proc = (RubyProc)args[1].convertToType(recv.getRuntime().getClass("Proc"), 0, "to_proc", true);
+        }
+        
+        recv.getRuntime().getGlobalVariables().setTraceVar(var, proc);
+        
+        return recv.getRuntime().getNil();
+    }
+
+    public static IRubyObject untrace_var(IRubyObject recv, IRubyObject[] args, Block block) {
+        if (args.length == 0) throw recv.getRuntime().newArgumentError(0, 1);
+        String var = null;
+        
+        if (args.length >= 1) {
+            var = args[0].toString();
+        }
+        
+        if (var.charAt(0) != '$') {
+            // ignore if it's not a global var
+            return recv.getRuntime().getNil();
+        }
+        
+        if (args.length > 1) {
+            ArrayList success = new ArrayList();
+            for (int i = 1; i < args.length; i++) {
+                if (recv.getRuntime().getGlobalVariables().untraceVar(var, args[i])) {
+                    success.add(args[i]);
+                }
+            }
+            return RubyArray.newArray(recv.getRuntime(), success);
+        } else {
+            recv.getRuntime().getGlobalVariables().untraceVar(var);
+        }
+        
+        return recv.getRuntime().getNil();
+    }
+
     public static IRubyObject singleton_method_added(IRubyObject recv, IRubyObject symbolId, Block block) {
         return recv.getRuntime().getNil();
     }
@@ -817,7 +950,7 @@ public class RubyKernel {
     
     
     public static RubyProc proc(IRubyObject recv, Block block) {
-        return recv.getRuntime().newProc(true, block);
+        return recv.getRuntime().newProc(Block.Type.LAMBDA, block);
     }
 
     public static IRubyObject loop(IRubyObject recv, Block block) {
@@ -867,8 +1000,21 @@ public class RubyKernel {
         switch(cmd) {
         
         // implemented commands
+        case 'C': // ?C  | Time    | Last change time for file1
+        case 'd': // ?d  | boolean | True if file1 exists and is a directory
+        case 'e': // ?e  | boolean | True if file1 exists
         case 'f':
         case 'M':
+        case 's': // ?s  | int/nil | If file1 has nonzero size, return the size,
+                  //     |         | otherwise return nil
+        case 'z': // ?z  | boolean | True if file1 exists and has a zero length
+        case '=': // ?=  | boolean | True if the modification times of file1
+                  //     |         | and file2 are equal
+        case '<': // ?<  | boolean | True if the modification time of file1
+                  //     |         | is prior to that of file2
+        case '>': // ?>  | boolean | True if the modification time of file1
+                  //     |         | is after that of file2
+        case '-': // ?-  | boolean | True if file1 and file2 are identical
             break;
 
         // unimplemented commands
@@ -879,9 +1025,6 @@ public class RubyKernel {
         case 'A': // ?A  | Time    | Last access time for file1
         case 'b': // ?b  | boolean | True if file1 is a block device
         case 'c': // ?c  | boolean | True if file1 is a character device
-        case 'C': // ?C  | Time    | Last change time for file1
-        case 'd': // ?d  | boolean | True if file1 exists and is a directory
-        case 'e': // ?e  | boolean | True if file1 exists
         case 'g': // ?g  | boolean | True if file1 has the \CF{setgid} bit
         case 'G': // ?G  | boolean | True if file1 exists and has a group
                   //     |         | ownership equal to the caller's group
@@ -896,8 +1039,6 @@ public class RubyKernel {
                   //     |         | uid/gid of the caller
         case 'R': // ?R  | boolean | True if file is readable by the real
                   //     |         | uid/gid of the caller
-        case 's': // ?s  | int/nil | If file1 has nonzero size, return the size,
-                  //     |         | otherwise return nil
         case 'S': // ?S  | boolean | True if file1 exists and is a socket
         case 'u': // ?u  | boolean | True if file1 has the setuid bit set
         case 'w': // ?w  | boolean | True if file1 exists and is writable by
@@ -907,20 +1048,7 @@ public class RubyKernel {
                   //     |         | the effective uid/gid
         case 'X': // ?X  | boolean | True if file1 exists and is executable by
                   //     |         | the real uid/gid
-        case 'z': // ?z  | boolean | True if file1 exists and has a zero length
-                  //
-                  //        Tests that take two files:
-                  //
-        case '-': // ?-  | boolean | True if file1 and file2 are identical
-        case '=': // ?=  | boolean | True if the modification times of file1
-                  //     |         | and file2 are equal
-        case '<': // ?<  | boolean | True if the modification time of file1
-                  //     |         | is prior to that of file2
-        case '>': // ?>  | boolean | True if the modification time of file1
-                  //     |         | is after that of file2
-            
             throw runtime.newArgumentError("unimplemented command ?"+(char)cmd);
-            
         default:
             // matches MRI message
             throw runtime.newArgumentError("unknown command ?"+(char)cmd);
@@ -947,16 +1075,49 @@ public class RubyKernel {
         
         File pwd = new File(runtime.getCurrentDirectory());
         File file1 = new File(pwd,args[1].convertToString().toString());
+        File file2 = null;
         Calendar calendar = null;
                 
         switch (cmd) {
+        case 'C': // ?C  | Time    | Last change time for file1
+            return runtime.newFixnum(file1.lastModified());
+        case 'd': // ?d  | boolean | True if file1 exists and is a directory
+            return runtime.newBoolean(file1.isDirectory());
+        case 'e': // ?e  | boolean | True if file1 exists
+            return runtime.newBoolean(file1.exists());
         case 'f': // ?f  | boolean | True if file1 exists and is a regular file
-            return RubyBoolean.newBoolean(runtime, file1.isFile());
+            return runtime.newBoolean(file1.isFile());
         
         case 'M': // ?M  | Time    | Last modification time for file1
             calendar = Calendar.getInstance();
             calendar.setTimeInMillis(file1.lastModified());
             return RubyTime.newTime(runtime, calendar);
+        case 's': // ?s  | int/nil | If file1 has nonzero size, return the size,
+                  //     |         | otherwise return nil
+            long length = file1.length();
+
+            return length == 0 ? runtime.getNil() : runtime.newFixnum(length);
+        case 'z': // ?z  | boolean | True if file1 exists and has a zero length
+            return runtime.newBoolean(file1.exists() && file1.length() == 0);
+        case '=': // ?=  | boolean | True if the modification times of file1
+                  //     |         | and file2 are equal
+            file2 = new File(pwd, args[2].convertToString().toString());
+            
+            return runtime.newBoolean(file1.lastModified() == file2.lastModified());
+        case '<': // ?<  | boolean | True if the modification time of file1
+                  //     |         | is prior to that of file2
+            file2 = new File(pwd, args[2].convertToString().toString());
+            
+            return runtime.newBoolean(file1.lastModified() < file2.lastModified());
+        case '>': // ?>  | boolean | True if the modification time of file1
+                  //     |         | is after that of file2
+            file2 = new File(pwd, args[2].convertToString().toString());
+            
+            return runtime.newBoolean(file1.lastModified() > file2.lastModified());
+        case '-': // ?-  | boolean | True if file1 and file2 are identical
+            file2 = new File(pwd, args[2].convertToString().toString());
+
+            return runtime.newBoolean(file1.equals(file2));
         default:
             throw new InternalError("unreachable code reached!");
         }
@@ -999,11 +1160,15 @@ public class RubyKernel {
             ceil = 0;
         } else if (args.length == 1) {
             if (args[0] instanceof RubyBignum) {
-                byte[] bytes = new byte[((RubyBignum) args[0]).getValue().toByteArray().length - 1];
+                byte[] bigCeilBytes = ((RubyBignum) args[0]).getValue().toByteArray();
+                BigInteger bigCeil = new BigInteger(bigCeilBytes).abs();
                 
-                runtime.getRandom().nextBytes(bytes);
+                byte[] randBytes = new byte[bigCeilBytes.length];
+                runtime.getRandom().nextBytes(randBytes);
                 
-                return new RubyBignum(runtime, new BigInteger(bytes).abs()); 
+                BigInteger result = new BigInteger(randBytes).abs().mod(bigCeil);
+                
+                return new RubyBignum(runtime, result); 
             }
              
             RubyInteger integerCeil = (RubyInteger) args[0].convertToType(runtime.getClass("Integer"), MethodIndex.TO_I, "to_i", true);
@@ -1016,7 +1181,7 @@ public class RubyKernel {
             return RubyFloat.newFloat(runtime, runtime.getRandom().nextDouble()); 
         }
         if (ceil > Integer.MAX_VALUE) {
-            return runtime.newFixnum(runtime.getRandom().nextLong() % ceil);
+            return runtime.newFixnum(Math.abs(runtime.getRandom().nextLong()) % ceil);
         }
             
         return runtime.newFixnum(runtime.getRandom().nextInt((int) ceil));
@@ -1024,7 +1189,12 @@ public class RubyKernel {
 
     public static RubyBoolean system(IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = recv.getRuntime();
-        int resultCode = new ShellLauncher(runtime).runAndWait(args);
+        int resultCode;
+        try {
+            resultCode = new ShellLauncher(runtime).runAndWait(args);
+        } catch (Exception e) {
+            resultCode = 127;
+        }
         recv.getRuntime().getGlobalVariables().set("$?", RubyProcess.RubyStatus.newProcessStatus(runtime, resultCode));
         return runtime.newBoolean(resultCode == 0);
     }
