@@ -984,7 +984,7 @@ public class RubyModule extends RubyObject {
     public RubyClass defineClassUnder(String name, RubyClass superClazz, ObjectAllocator allocator) {
         IRubyObject type = getInstanceVariable(name);
 
-        if (type == null) {
+        if (type == null || (type instanceof RubyUndef)) {
             return getRuntime().defineClassUnder(name, superClazz, allocator, cref);
         }
 
@@ -994,7 +994,15 @@ public class RubyModule extends RubyObject {
             throw getRuntime().newNameError(name + " is already defined.", name);
         }
 
-        return (RubyClass) type;
+        RubyClass klazz = (RubyClass) type;
+
+        // If we define a class in Ruby, but later want to allow it to be defined in Java,
+        // the allocator needs to be updated
+        if (klazz.getAllocator() != allocator) {
+            klazz.setAllocator(allocator);
+        }
+    
+        return klazz;
     }
 
     public RubyModule defineModuleUnder(String name) {
