@@ -11,7 +11,7 @@
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
- * Copyright (C) 2006 Ola Bini <ola@ologix.com>
+ * Copyright (C) 2007 Ola Bini <ola.bini@gmail.com>
  * 
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -209,7 +209,7 @@ public class RubyYAML {
     }
 
     public static IRubyObject load(IRubyObject self, IRubyObject arg) {
-        IRubyObject io = arg;
+        IRubyObject io = check_yaml_port(arg);
         Scanner scn = null;
         if(io instanceof RubyString) {
             scn = new ScannerImpl(((RubyString)io).getByteList());
@@ -249,7 +249,7 @@ public class RubyYAML {
 
     public static IRubyObject load_documents(IRubyObject self, IRubyObject arg, Block block) {
         ThreadContext context = self.getRuntime().getCurrentContext();
-        IRubyObject io = arg;
+        IRubyObject io = check_yaml_port(arg);
         Scanner scn = null;
         if(io instanceof RubyString) {
             scn = new ScannerImpl(((RubyString)io).getByteList());
@@ -286,7 +286,7 @@ public class RubyYAML {
     public static IRubyObject dump_stream(IRubyObject self, IRubyObject[] args) {
         ThreadContext context = self.getRuntime().getCurrentContext();
         IRubyObject stream = self.getRuntime().getModule("YAML").getClass("Stream").callMethod(context, "new");
-        for(int i=0,j=args.length;i<j;) {
+        for(int i=0,j=args.length;i<j;i++) {
             stream.callMethod(context,"add", args[i]);
         }
         return stream.callMethod(context, "emit");
@@ -300,6 +300,23 @@ public class RubyYAML {
         return self.getRuntime().getNil();
     }
 
+    // prepares IO port type for load (ported from ext/syck/rubyext.c)
+    private static IRubyObject check_yaml_port(IRubyObject port) {
+        if (port instanceof RubyString) {
+            // OK
+        }
+        else if (port.respondsTo("read")) {
+            if (port.respondsTo("binmode")) {
+                ThreadContext context = port.getRuntime().getCurrentContext();
+                port.callMethod(context, "binmode");
+            }
+        }
+        else {
+            throw port.getRuntime().newTypeError("instance of IO needed");
+        }
+        return port;
+    }
+
     public static IRubyObject hash_to_yaml_node(IRubyObject self, IRubyObject arg) {
         ThreadContext context = self.getRuntime().getCurrentContext();
         return arg.callMethod(context,"map", new IRubyObject[]{self.callMethod(context, "taguri"),self,self.callMethod(context, "to_yaml_style")});
